#!/usr/bin/python
import sys
import os
from gwas import hapIO
from gwas import summary_stat as ss
from gwas import hapdata
import numpy as np
import subprocess
import tarfile

def simul_stats_one_rep_macld(outfile_name,rep,nb_seg,L,n,times,params,interval_list,size_list,prob_list,dmax,mac=1,mac_ld=10,prob_err=0,save_ms=False):
    '''
    Simulates one sample with ms and returns a vector of summary statistics
    If save_ms==True, also prints the raw ms output in outfile_name_repi_segj.ms for each segment j (where i is given by argument rep)
    Each allele is switched ( 0 becomes 1 and 1 becomes 0) with probability prob_err (default is 0).
    '''
    # build the root of the ms command
    N=params[2]
    theta=max(0,4*params[0]*N*L)
    rho=max(0,4*params[1]*N*L)
    cmd='ms/ms '+str(n)+' 1 -t '+str(theta)+' -r '+str(rho)+' '+str(L)+' -p 7'
    times=times/(4*N)
    for i in range(1,len(times)):
	cmd=cmd+' -eN '+str(times[i])+' '+str(params[i+2]/N)
    # simulate and summarize each segment
    count_list=[]
    pos_list=[]
    hap_list=[]
    pos2_list=[] # for ld
    hap2_list=[] # for ld
    nb_snp=0
    for i in range(nb_seg):
	# ensure that different seeds will be used for different segments
    	# seeds_ms=np.random.choice(100000,size=3,replace=False)
	seeds_ms=np.random.permutation(10000)
	seeds_ms=seeds_ms[0:3]
    	cmd2=cmd+' -seeds '+str(seeds_ms[0])+' '+str(seeds_ms[1])+' '+str(seeds_ms[2])
	# simulate sample with ms
	p = subprocess.Popen(cmd2, stdout=subprocess.PIPE, shell=True)
    	(output, err) = p.communicate()
	if not err == None:
	    print 'Error while running ms'
	else:
	    # print ms file if required
	    if save_ms==True:
		outfile=open(outfile_name+'_rep'+str(rep+1)+'_seg'+str(i+1)+'.ms','w')
		outfile.write(output)
            # store data and pre-analyse it     
            mydata=hapIO.parsems_onepop_nofile_fast('ms_rep',output)
	    if mydata.nsnp>0:
		if prob_err>0:
		    mydata.introduce_errors(prob_err)
                u=hapdata.Haplos_and_counts_fast(mydata,'pop1',mac=mac)
                pos_list.append(u[0])
                count_list.append(u[1])
                hap_list.append(u[2])
		nb_snp+=len(u[0])
		u=hapdata.Haplos_and_counts_fast(mydata,'pop1',mac=mac_ld)
		pos2_list.append(u[0])
                hap2_list.append(u[2])
	    else:
		print 'no segsite in segment', i+1, 'of replicate', rep+1
    geno_list=ss.hap_to_geno(hap_list)
    geno2_list=ss.hap_to_geno(hap2_list)
    # computation of summary statistics
    res_afs=ss.spatial_histo(pos_list,count_list,n/2)
    res_ld_zyg=ss.distrib_zyg_r2(pos2_list,geno2_list,interval_list)
    res_ibs=ss.ibs_quantiles_from_geno(size_list[0],pos_list,geno_list,prob_list,dmax)
    for m in size_list[1:]:
	res_ibs=np.concatenate((res_ibs,ss.ibs_quantiles_from_geno(m,pos_list,geno_list,prob_list,dmax)))
    return np.concatenate((np.array([np.float(nb_snp)/(L*nb_seg)],dtype='float'),res_afs[0],res_ld_zyg[0],res_ibs))

def comp_stats_one_rep_macld(outfile_name,rep,nb_seg,L,n,interval_list,size_list,prob_list,dmax,mac=1,mac_ld=10,prob_err=0):
    '''
    Reads nb_seg ms files and returns a vector of summary statistics
    The ms file for segment j is called outfile_name_repi_segj.ms (where i is given by argument rep)
    Summary statistics are computed for n randomly sampled sequences and the first nb_seg segments
    Each allele is switched ( 0 becomes 1 and 1 becomes 0) with probability prob_err (default is 0).
    '''
    # open tar archive
    mytar=tarfile.open(outfile_name+'.ms.tar.bz2','r:bz2')
    # simulate and summarize each segment
    count_list=[]
    pos_list=[]
    hap_list=[]
    pos2_list=[] # for ld
    hap2_list=[] # for ld
    nb_snp=0
    for i in range(nb_seg):
	try:
	    # store data and pre-analyse it
	    output=mytar.extractfile(outfile_name+'_rep'+str(rep+1)+'_seg'+str(i+1)+'.ms').read()
	    mydata=hapIO.parsems_onepop_nofile_fast('ms_rep',output,n2=n)         
	    if mydata.nsnp>0:
		if prob_err>0:
		    mydata.introduce_errors(prob_err)
                u=hapdata.Haplos_and_counts_fast(mydata,'pop1',mac=mac)
                pos_list.append(u[0])
                count_list.append(u[1])
                hap_list.append(u[2])
		nb_snp+=len(u[0])
		u=hapdata.Haplos_and_counts_fast(mydata,'pop1',mac=mac_ld)
		pos2_list.append(u[0])
                hap2_list.append(u[2])
	    else:
		print 'No segsite in segment', i+1, 'of replicate', rep+1
	except IOError:
	    print 'File '+outfile_name+'_rep'+str(rep+1)+'_seg'+str(i+1)+'.ms does not exist'
    mytar.close()
    geno_list=ss.hap_to_geno(hap_list)
    geno2_list=ss.hap_to_geno(hap2_list)
    # computation of summary statistics
    res_afs=ss.spatial_histo(pos_list,count_list,n/2)
    res_ld_zyg=ss.distrib_zyg_r2(pos2_list,geno2_list,interval_list)
    res_ibs=ss.ibs_quantiles_from_geno(size_list[0],pos_list,geno_list,prob_list,dmax)
    for m in size_list[1:]:
	res_ibs=np.concatenate((res_ibs,ss.ibs_quantiles_from_geno(m,pos_list,geno_list,prob_list,dmax)))
    return np.concatenate((np.array([np.float(nb_snp)/(L*nb_seg)],dtype='float'),res_afs[0],res_ld_zyg[0],res_ibs))

